function varargout = Fourier_trunc(xdata,spectrum,sigma)

% Data manipulation technique to smooth data
%
% Usage:
%   spectrum_denoised =  Fourier_Trunc(xdata,spectrum,sigma)
%
%
%
%   spectrum_denoised              - vector
%                                    smoothed data
%
%   xdata                          - vector
%                                    magnetic field vector
%
%   spectrum                       - vector
%                                    original data set
%
%   sigma                          - vector
%                                    numeric, approximate standard
%                                    deviation of the signal
%
%
% The factor of denoiselevel is taken and a Gaussian distribution is
% is assumed in the magnetic field space and in the conjugated Fourier
% space. The invese standard deviation is calculated and 3 sigma are taken
% as threshhold value in the conjugated Fourier space. Higher contributions
% are set to zero. The real part of the ,symmetric inverse Fourier transform
% of this is returned.
%
% (c) 2018, Stephan Rein


%Check if the vector is transposed
dim = size(spectrum);
if dim(1) == 1
    spectrum =  spectrum';
end

%Determine standard deviation in Fourier space
denoiselevel = 3*(1/(sigma*2));
%Building X-axis of the Fourier transform
Fs = 1/(xdata(2)-xdata(1));
v= (0:1:length(spectrum)-1)*Fs/length(spectrum);

% Allocated return vector (default)
spectrum_denoised = spectrum;
%Use heayside function in fourier space
heaviside = denoiselevel;
if heaviside ~= 0
    %Less than 0.001  makes no sense for truncation
    if heaviside < 0.001
        heaviside = 0.001;
    end
    Y = fft(spectrum);    
    %Fill with one zero if its a odd number
    if mod(length(Y),2) == 1
        Y = [Y(1:round(length(Y)/2)); 0; Y(round(length(Y)/2)+1:end)];
    end   
    for x=1:length(Y);
        if v(x) > heaviside
            Y(x) = 0;
        end
    end
    Y = ifft(Y,'symmetric');
    spectrum_denoised = real(Y);
end
varargout{1} =  spectrum_denoised;
return
