function signal = Gaussian_smooth(xdata, signal, FWHM)

% Gaussian smoothing of an EPR signal
%
%   signal = Gaussian_smooth(xdata,signal, FWHM)
%
%   xdata                   - Vector
%                             Magnetic field vector
%
%   signal                  - Vector
%                             Singal vector
%
%   FWHM                    - Float
%                             Value of the full width at half maximum
%                             (FWHM) used for the Gaussian function.
%
%   OUTPUT
%   signal                  - vector
%                             Smoothed data
%
%
% Direct vector convolution of the signal with a Gaussian function using a
% standard deviation std. The passed asrgument std is converted from a FWHM
% paramter to the standard deviation of the Gaussian function. The passed
% standard-deviation should not be larger than 0.5*FWHM.
%
% Copyright (c) 2018, Stephan Rein

xdata_mid = (max(xdata)+min(xdata))/2;
std = FWHM/sqrt(2*log(2));
G = (1/(std*sqrt(2*pi)))*exp(-2*(xdata-xdata_mid).^2/std^2);
signal = conv(G,signal,'same');

