function [realsignal,imagsignal] = Hilbert_transform(realsignal)

% Usage: Hilbert transform of a real-valued EPR signal
%   
%   INPUT
%   realsignal              - Vector
%                             Real EPR data vector
%
%   OUTPUT
%   realsignal              - Vector
%                             Real EPR data vector
%
%   imagsignal              - Vector
%                             Imaginary EPR data output
%
% The alogrithm is based on:
% Marple, S. L. IEEE Transactions on Signal Processing. Vol. 47, 1999,
% 2600-2603

% Check if the vector needs to be transposed
dim = size(realsignal);
if dim(1) == 1
    realsignal =  realsignal';
end

% Fourier transform of the real input data
real_tmp = fft(realsignal);

% Get signum permutations (rotation in complex plane)
h = zeros(length(real_tmp),1);
x = length(real_tmp);
for i=1:x
    if i >= 2 && i  <= x/2
        h(i) = 2;
    elseif i >= x/2+2
        h(i) = 0;
    else
        h(i) = 1;
    end
end

% Convolution with signum permutations
real_tmp = real_tmp.*h;

%Inverse Fourier transform
imagsignal = imag(ifft(real_tmp));
realsignal = real(ifft(real_tmp));
if dim(1) == 1
    realsignal =  realsignal';
    imagsignal = imagsignal';
end
