function FitOpt = Input_process1(FitOpt, Intensity, number_of_spc)

% Function 1 for processing the user-defined input
%
%**************************************************************************
% Cutting and Weighting
%**************************************************************************
if isfield(FitOpt,'algorithm')
else
    FitOpt.algorithm = 'trust-region-reflective';
end

if isfield(FitOpt,'cuttingup')
    FitOpt.cuttingup = FitOpt.cuttingup;
else
    FitOpt.cuttingup(1:number_of_spc) = 10000000;
end

if isfield(FitOpt,'cuttingdown')
    FitOpt.cuttingdown = FitOpt.cuttingdown;
else
    % Cutting parameter defaults
    FitOpt.cuttingdown(1:number_of_spc) = 0;
end

% Weighting factors
if isfield(FitOpt,'weightingfactor')
    for i = 1:number_of_spc
        %Weighting area for least-square fitting (optional)
        if strcmp(FitOpt.weightingfactor,'auto')
            Intensity_tmp = Intensity{i}/max(abs(Intensity{i}));
            [~, detail] = discrete_wt(Intensity_tmp, 1);
            FitOpt.weighting_factor(i) = 1.0/(std(detail)*length(Intensity_tmp));
        else
            FitOpt.weighting_factor(i) =  FitOpt.weightingfactor(i);
        end
        
        if isfield(FitOpt,'weightingdown')
            FitOpt.weighting_area_d(i) = FitOpt.weightingdown(i);
        else
            %default weighting parameter
            FitOpt.weighting_area_d(i) = 0;
        end
        if isfield(FitOpt,'weightingup')
            FitOpt.weighting_area_u(i) = FitOpt.weightingup(i);
        else
            % Cutting parameter defaults
            FitOpt.weighting_area_u(i) = 1000000000000;
        end
    end
else
    FitOpt.weighting_factor(1:number_of_spc) =  1;
end

if isfield(FitOpt,'AbsScale')
else
    FitOpt.AbsScale = false;
end

if isfield(FitOpt,'maxAbs')
else
    FitOpt.maxAbs = false;
end

if isfield(FitOpt, 'diff_isotopes')
    FitOpt.diff_isotopes = cellstr(FitOpt.diff_isotopes);
end

if isfield(FitOpt, 'LHS')
else
    FitOpt.LHS = true;
end

%**************************************************************************
% Other user-defined options
%**************************************************************************

% Check number of deterministic steps
if isfield(FitOpt,'nIter')
else
    FitOpt.nIter = 30;
end

%Check if equivalent HF couplings are needed
if isfield(FitOpt,'equivHFcoupling')
   FitOpt.isequivHFcoupling = true;
else
    FitOpt.isequivHFcoupling = false;   
end

% Check number of Monte Carlo sampling if Monte Carlo is used
if FitOpt.LHS == true
    if isfield(FitOpt,'nTrials')
        FitOpt.maxn_iter = FitOpt.nTrials;
    else
        FitOpt.nTrials = 2;
        FitOpt.maxn_iter = FitOpt.nTrials;
    end
else
    if isfield(FitOpt,'nTrials')
        FitOpt.maxn_iter = FitOpt.nTrials;
    else
        FitOpt.nTrials = 2 ;
        FitOpt.maxn_iter = FitOpt.nTrials;
    end
end


% Check if  FitOpt.smartGrid is on
if isfield(FitOpt, 'smartGrid')
    %Define coarse grid
    if FitOpt.smartGrid == true
        FitOpt.nIter = round(FitOpt.nIter/2);
        if FitOpt.nIter == 0
            FitOpt.nIter = 1;
        end
    end
else
     FitOpt.smartGrid = 0;
end

% Check if  FitOpt.smartGrid is on
if isfield(FitOpt,'fastGrid')
    %Define coarse grid
    if FitOpt.fastGrid == true || FitOpt.fastGrid > 0
        FitOpt.factor = 1;
        if  FitOpt.smartGrid ~= true
            FitOpt.smartGrid= true;
            FitOpt.nIter = round(FitOpt.nIter/2);
        end
        if FitOpt.nIter == 0
            FitOpt.nIter = 1;
        end
        if FitOpt.fastGrid == true || FitOpt.fastGrid == 1
            FitOpt.factor = 1;
        elseif FitOpt.fastGrid > 1
            FitOpt.factor = round(FitOpt.fastGrid);
        end
        FitOpt.fastGrid = true;
    end
else
    FitOpt.fastGrid = 0;
end

% Check if plot is updated
if isfield(FitOpt,'plotoff')
else    
    FitOpt.plotoff = false;
end

% Check if final plot should be created
if isfield(FitOpt,'final_plot')
else    
    FitOpt.final_plot = false;
end


% Check if simulation results should be printed
if isfield(FitOpt,'verbose')
else    
    FitOpt.verbose = 'iter';
end