function [FitOpt,startpoints_cell, easyspinfun] = Input_process2(FitOpt, Exp, easyspinfun)

% Function 2 for processing the user-defined input
%
% Check if  FitOpt.smartGrid is on
if isfield(FitOpt,'TolFun')
else
    FitOpt.TolFun = 1.0e-5;
end

% Define static
FitOpt.sameHFcoupling = false;

%Check paralellization
fprintf('\n')
if hasParallelComputingToolbox()
    disp('Parallel Computing Toolbox is available:');
else
    disp('Parallel Computing Toolbox is not available.');
end

% Check if parallelization is requested
if isfield(FitOpt,'useParallel') && hasParallelComputingToolbox()
    FitOpt.useParallel = FitOpt.useParallel;
    if FitOpt.useParallel== true
        poolobj = gcp('nocreate'); % If no pool, do not create new one .
        if isempty(poolobj)
            poolobj =parpool;
            poolsize = poolobj.NumWorkers;
        else
            poolsize = poolobj.NumWorkers;
        end
        FitOpt.cores = poolsize;
        startpoints_cell = cell(FitOpt.cores,1);
        fprintf('Parallel computing is switched on using %i',FitOpt.cores);
        fprintf(' CPUs\n');
        % Increase the sampling points of the hypercube of they are not
        % a multiple of the FitOpt.cores
        if mod(FitOpt.maxn_iter,FitOpt.cores) ~= 0
            FitOpt.maxn_iter = FitOpt.maxn_iter+FitOpt.cores-mod(FitOpt.maxn_iter,FitOpt.cores);
            FitOpt.nTrials = FitOpt.maxn_iter;
        end
    else
        fprintf('No parallel computing\n')
        FitOpt.cores = 1;
    end
else
    FitOpt.useParallel = false;
    fprintf('No parallel computing\n')
    FitOpt.cores = 1;
    startpoints_cell = cell(FitOpt.cores,1);
end

%Check if optimization toolbox is available
fprintf('\n')
if hasOptimizationToolbox()
    disp('Optimization Toolbox is available:');
    FitOpt.algor = which_algorithm(FitOpt.algorithm);
else
    disp('Optimization Toolbox is not available.');
    FitOpt.algorithm = 6;
    FitOpt.algor = which_algorithm(FitOpt.algorithm);
end

%str2func (Define which EasySpin script should be called)
%Check if Exp is a cell array
if ~iscell(easyspinfun)
    easyspinfun_tmp = easyspinfun;
    clear easyspinfun;
    for i =1:length(Exp)
        easyspinfun{i} = easyspinfun_tmp;
    end
end

%Define string for axis labeleing
FitOpt.xlab = cell(length(Exp),1);
for i =1:length(Exp)
    if strcmp(easyspinfun{i},'salt')
        FitOpt.xlab{i} = '{\it \omega} / MHz';
    else
        FitOpt.xlab{i} = '{\it B}_0 / mT';
    end
end