function X = LHS(np, nv, seed, ns)
% Latin hypercube sampler
% nv – number of variables in the LH
% seed – initial seed design (points within 1 and ns)
% ns – number of points in the seed design
% outputs: X – Latin hypercube created using the translational
% define the size of the TPLHD to be created first

nd = ( np/ns )^( 1/nv ); % number of divisions, nd
ndStar = ceil( nd );
if (ndStar > nd)
nb = ndStar^nv; % it is necessary to create a bigger TPLHD
else
nb = np/ns; % it is NOT necessary to create a bigger TPLHD
end
npStar = nb*ns; % size of the TPLHD to be created first

% reshape seed to properly create the first design
seed = reshapeSeed(seed , ns, npStar, ndStar, nv);
% create TPLHD with npStar points
X = createTPLHD(seed, npStar, ndStar, nv);
% resize TPLH if necessary npStar > np;
if (npStar > np)
X = resizeTPLHD(X, npStar, np, nv);
end
%Scale between 0 and 1
X = X./np;
end



