function [bestparameter,lsqbest] = SpecProFi_kernel(easyspinfun,Intexp_tmp, ...
                                                    Sys,Vary,Exp,Opt,FitOpt)

%% Main function kernel of SpecProFi
%All informations are noted in the SpecProFi function

%Close all current windows
close all;

%Current version number
version = 'SpecProFi 1.0.2';

%**************************************************************************
%% PARAMETER SETUP AND ALLOCATION
%**************************************************************************

%Check paralellization
fprintf('\n\n\n');
fprintf('\n\n**********************************************************************\n');
fprintf('******************************SpecProFi******************************\n\n\n');
help('SpecProFi')
fprintf('\n  Version: %s',version);
fprintf('\n\n\n');
fprintf('\n**********************************************************************\n');
fprintf('****************       USER DEFINED SETTINGS       *******************\n');
fprintf('**********************************************************************\n\n');

FitOpt.Intexp_tmp = Intexp_tmp;
if ~iscell(FitOpt.Intexp_tmp)
    number_of_spc = 1;
    FitOpt.Intexp_tmp_tmp{1} = FitOpt.Intexp_tmp;
    clear FitOpt.Intexp_tmp;
    FitOpt.Intexp_tmp = FitOpt.Intexp_tmp_tmp;
else
    number_of_spc = length(FitOpt.Intexp_tmp);
end

% Allocations
FitOpt.weighting_factor = zeros(number_of_spc,1);
FitOpt.weighting_area_d = zeros(number_of_spc,1);
FitOpt.weighting_area_u = zeros(number_of_spc,1);

%**************************************************************************
%% Convert everything into cell arrays
%**************************************************************************
if isstruct(Sys)
    newSys{1} = Sys;
    clear Sys;
    Sys = newSys;
end
%Check if Exp is a cell array
if isstruct(Vary)
    newVary{1} = Vary;
    clear Vary;
    Vary = newVary;
end
%Check if Exp is a cell array
if isstruct(Exp)
    newExp{1} = Exp;
    clear Exp;
    Exp = newExp;
end

if isfield(FitOpt,'nonuniform_field')
    i = 1;
    Field = FitOpt.nonuniform_field;
    Range = linspace(Exp{i}.Range(1), Exp{i}.Range(2), 2048);
    FitOpt.Intexp_tmp{i} = interp1(Field, FitOpt.Intexp_tmp{i}, Range);
end

%**************************************************************************
%% Process input settings
%**************************************************************************
FitOpt = Input_process1(FitOpt, FitOpt.Intexp_tmp, number_of_spc);
[FitOpt,startpoints_cell, easyspinfun] = Input_process2(FitOpt, Exp, easyspinfun);
%Silence the ouput
Opt.Verbosity = 0;
%default setup for normalization
Sysst = Sys;

%Correct number of points:
for i = 1:length(Exp)
    if i <= number_of_spc
        Exp_tmp = Exp{i};
        Exp_tmp.nPoints = length(FitOpt.Intexp_tmp{i});
        Exp{i} = Exp_tmp;
    else
        Exp_tmp = Exp{i};
        Exp_tmp.nPoints = length(FitOpt.Intexp_tmp{i});
        Exp{i} = Exp_tmp;
    end
end
Sys_static = Sys;

%**************************************************************************
%% FIRST GUESS, CUTTING, WEIGHTING AND NORMALIZATION
%**************************************************************************
FitOpt = initial_guess(Sys, Exp,Opt, FitOpt, easyspinfun, number_of_spc);

% Redefinition of points:
for i = 1:length(Exp)
    if i <= number_of_spc
        Exp_tmp = Exp{i};
        Exp_tmp.nPoints = length(FitOpt.Intexp_tmp{i});
        Exp{i} = Exp_tmp;
    else
        Exp_tmp = Exp{i};
        Exp_tmp.nPoints = length(FitOpt.Intexp_tmp{i});
        Exp{i} = Exp_tmp;
    end
end

%Correct the field range:
for i = 1:length(Exp)
    Exp_tmp = Exp{i};
    Exp_tmp.Range(1) = min(FitOpt.Field_exp_tmp{i});
    Exp_tmp.Range(2) = max(FitOpt.Field_exp_tmp{i});
    Exp{i} = Exp_tmp;
end

%Update the static structures
Expst = Exp;
Sys = Sys_static;

Frequency_exp = [FitOpt.Field_exp_tmp{1:number_of_spc}];
Int = [FitOpt.Int_tmp{1:length(1:number_of_spc)}];
Intexp =[FitOpt.Intexp_tmp{1:number_of_spc}];

%Initial best parameters
bestparameter = Sys;
% %PLOT (always updated, therefore kept as handel)
[h, pk] = initial_plot(FitOpt, FitOpt.Field_exp_tmp,...
                      FitOpt.Intexp_tmp, number_of_spc);

%least squares
lsqbest = sum((Int-Intexp).^2);
lsqbest_tmp = lsqbest;

%counter for  FitOpt.smartGrid
q = 1;
bestparameter_cell = cell(FitOpt.maxn_iter);

%MONTE CARLO OR LATIN HYPERCUBE INITALIZATION
LH  = parameter_sampling(FitOpt, Vary);

%**************************************************************************
%% Give user imformation before starting
%**************************************************************************
print_user_information(FitOpt)
%% STARTING ITERATION LOOP
fitparam_cell = cell(FitOpt.maxn_iter);
lsqvalues = zeros(FitOpt.maxn_iter,1);
tic
for number_of_iterations = 1:FitOpt.maxn_iter
    t = number_of_iterations; %Just a shorter variable name
    %User information
    fprintf('\nNumber of sampling: %i ',number_of_iterations);
    fprintf('of %i',FitOpt.maxn_iter);
    fprintf('\n\n');
    
    % REFRESH SYS
    Sys = Sysst;
    Exp = Expst;
    [Sys, Exp] = varyParameters(Sys,Vary,LH(number_of_iterations,:),Exp,FitOpt);
    %Start deterministic part of the algorithm
    %Generate input vector
    modulo_iter = mod(t,FitOpt.cores)+1;
    [startpoints,bounderies] = generateOptVector(Sys,Vary,Sysst,Exp,Expst, FitOpt);
    %startpoints_cell = cell(2,1);
    startpoints_cell{modulo_iter} = startpoints;
    %Make it numerically stabel
    bounderies(1,:) = bounderies(1,:)-0.000000000001;
    bounderies(2,:) = bounderies(2,:)+0.000000000001;
    
    %LSQ-Curvefit or fmincon options
    options = define_algorithm_options(FitOpt.algor, FitOpt.nIter,...
                                       FitOpt.TolFun, FitOpt.verbose);
    
    %NONLINEAR OPTIMIZATION PARALLEL COMPUTING OR NOT
    if FitOpt.useParallel == true
        if mod(t,FitOpt.cores) == 0
            parfor j =1:FitOpt.cores
                fitparam_t{j} = optimize(Sys, Vary, Exp, Opt, ...
                    startpoints_cell{j}, bounderies, Frequency_exp,...
                    options, Intexp, FitOpt,h,...
                    easyspinfun, FitOpt.algor,  number_of_spc)
                fprintf('\nSampling %i', j+t-FitOpt.cores);
                fprintf(' done.....\n');
            end
        end
    else
        fitparam_t{1} = optimize(Sys,Vary,Exp,Opt,startpoints_cell{1}, ...
            bounderies,Frequency_exp,options, Intexp, FitOpt,h,...
            easyspinfun, FitOpt.algor,number_of_spc);
    end
    if mod(t,FitOpt.cores) == 0
        for j = 1:FitOpt.cores
            fitparam = fitparam_t{j};
            [Sys,Exp] = generateStruc(Vary,Sys,Exp,fitparam, FitOpt);
            % Simulation kernel
            [FitOpt.Field_exp_tmp, FitOpt.Int_tmp] = ...
                simulation_kernel(easyspinfun,Sys, Exp, FitOpt, ...
                                  Opt, FitOpt.weighting,number_of_spc);      
            Frequency_exp = [FitOpt.Field_exp_tmp{1:number_of_spc}];
            Int = [FitOpt.Int_tmp{1:number_of_spc}];          
            %Calculate least-squares
            lsqmontecarlo = sum((Int-Intexp).^2);
            %**********************************************************************
            % Switch on smart grid
            %**********************************************************************
            if  FitOpt.smartGrid == 1
                finegrid = false;
                fitparam_cell{number_of_iterations} = fitparam;
                bestparameter_cell{1} = startpoints;
                
                %Check if there is an improvement of the least-squares deviation
                if lsqmontecarlo < lsqbest_tmp
                    q = q+1;
                    finegrid = true;
                    bestparameter_cell{q} = fitparam_cell{number_of_iterations};
                    lsqbest_tmp = lsqmontecarlo;
                else
                    variationdiff= abs(bounderies(2,:)-bounderies(1,:))';
                    for pa = 1:q
                        norm_tmp = norm(abs(bestparameter_cell{pa} ...
                            -fitparam_cell{number_of_iterations})./variationdiff);
                        if pa == 1
                            bestnorm = norm_tmp;
                        else
                            if norm_tmp < bestnorm
                                bestnorm = norm_tmp;
                            end
                        end
                    end
                    
                    if bestnorm > length(fitparam)/20 && lsqmontecarlo-lsqbest_tmp < lsqbest_tmp
                        q = q+1;
                        bestparameter_cell{q} = fitparam_cell{number_of_iterations};
                        finegrid = true;
                    end
                end
                %Start deterministic part of the FitOpt.algorithm
                if finegrid == true
                    fprintf('\nAdditional optimization is switched on\n')
                    options = define_algorithm_options(FitOpt.algor, ...
                                                       FitOpt.nIter, ...
                                                       FitOpt.TolFun, ...
                                                       FitOpt.verbose);
                    if FitOpt.fastGrid == true
                        fprintf('Fastgrid:\n')
                        algor_tmp = FitOpt.algor;
                        if isfield(FitOpt, 'FGalgorithm')
                            FitOpt.algor = which_algorithm(FitOpt.FGalgorithm);
                        else
                            FitOpt.algor = 5;
                        end
                        options = optimset('MaxIter', FitOpt.factor, ...
                                           'TolFun', FitOpt.TolFun,...
                                           'MaxFunEvals', 15000,...
                                           'Display', FitOpt.verbose,...
                                           'UseParallel','never');
                    end
                    %NONLINEAR OPTIMIZATION
                    startpoints_tmp = fitparam;
                    fitparam = optimize(Sys,Vary,Exp,Opt,startpoints_tmp,...
                        bounderies,Frequency_exp,options, Intexp, FitOpt,h,...
                        easyspinfun, FitOpt.algor,number_of_spc);
                    %Switch back to user option
                    if FitOpt.fastGrid == true
                        FitOpt.algor = algor_tmp;
                    end             
                    %Generate Sys with optimized Parameters
                    [Sys,Exp] = generateStruc(Vary,Sys,Exp,fitparam, FitOpt);             
                    % Simulation kernel
                    [FitOpt.Field_exp_tmp, FitOpt.Int_tmp] =...
                              simulation_kernel(easyspinfun,Sys, Exp, FitOpt, ...
                                                Opt, FitOpt.weighting,number_of_spc);
                    Frequency_exp = [FitOpt.Field_exp_tmp{1:number_of_spc}];
                    Int = [FitOpt.Int_tmp{1:number_of_spc}];             
                    %Calculate least-squares
                    lsqmontecarlo = sum((Int-Intexp).^2);
                end
            end
            %Save the vector of least-squares
            lsqvalues(number_of_iterations+j-FitOpt.cores) = lsqmontecarlo;          
            %Log least-squares
            if  number_of_iterations==1 || (number_of_iterations==FitOpt.cores && j==1)
                fileID = fopen('Log_least_squares.txt','w');
                fprintf(fileID,'%f',lsqmontecarlo);
                fprintf(fileID,'\n');
                fclose(fileID);
                fileID = fopen('Log_Best_Parameters.txt','w');
                fprintf(fileID,'\n');
                fclose(fileID);
            else
                fileID = fopen('Log_least_squares.txt', 'a');
                fprintf(fileID,'%f',lsqmontecarlo);
                fprintf(fileID,'\n');
                fprintf(fileID,'');
                fclose(fileID);
            end
            %Check if there is an improvement of the least-squares deviation
            if lsqmontecarlo < lsqbest
                bestparameter = Sys;
                lsqbest=lsqmontecarlo;
                for ms = 1:number_of_spc
                    refreshPlot(pk{ms},FitOpt.Int_tmp{ms},FitOpt.Field_exp_tmp{ms});
                end
                %******************************************************************
                % Write output file for best current parameter set
                %******************************************************************
                write_output_files(bestparameter,version,Exp,Opt, ...
                                   Vary,FitOpt,Expst,Sysst,lsqbest,q,clock)
            end
            %Predict time when the program finishes
            if number_of_iterations >= 2 && j==1
                elapsed = toc;
                try
                    predict_time(elapsed, FitOpt.maxn_iter,number_of_iterations)
                catch ME
                    warning('Execption raised. No time prediction possible\n')
                    fprintf(ME.identifier)
                end
            end
        end
    end
end

%Best parameters shown on console
fprintf('Best least-square deviation = %f',lsqbest);
fprintf('\n\n');

%% CREATE FINAL OUTPUT
%Use timecode to create unique filename
long_n = clock;
outputstr = '_';
if isfield(FitOpt, 'filename')
    outputstr = strcat(strcat('_', FitOpt.filename), '_');
end
for i = 1:1:5
    outputstr = strcat(outputstr,num2str(long_n(i)));
    if i < 5
        outputstr =  strcat(outputstr,'_');
    end
end
final_output_files(outputstr)

%% Make final plot if wished
make_final_plot(FitOpt, Exp, Opt, easyspinfun, bestparameter, ...
    number_of_spc, lsqvalues, outputstr)
close all;