% Fitting set up for nonlinear optimization
function fitFunction = fitsimFunction(Sys,Vary,Exp,Opt,startpoints,FitOpt,h,...
                                       easyspinfun,number_of_spc)
m=0;
for spinSys = 1:length(Vary)
    Sys_tmp = Sys{spinSys};
    Vary_tmp = Vary{spinSys};
    nn=0;
    
    parameters = {'g','gFrame','AFrame','D', 'Q','QFrame', 'DFrame', ...
        'AStrain','lwEndor','HStrain','J','ee', 'eeD', 'nn','Abund', ...
        'gStrain','DStrain', 'weight','lwpp','lw','logtcorr'};
    for p = 1:length(parameters)
        if isfield(Vary_tmp,parameters{p})
            sA = size(Vary_tmp.(parameters{p}));
            if sA(1) == 1
                for n=1:(sA(1)*sA(2))
                    nn =nn+1;
                    Sys_tmp.(parameters{p})(n) = startpoints(nn+m);
                end
            elseif sA(1) > 1
                for n=1:(sA(1))
                    for k=1:(sA(2))
                        nn =nn+1;
                        Sys_tmp.(parameters{p})(n,k) = startpoints(nn+m);
                    end
                end
            end
        end
    end
    
    if isfield(Vary_tmp,'A')
        sA = size(Vary_tmp.A);
        if sA(1) == 1 && FitOpt.sameHFcoupling == false
            for n=1:(sA(1)*sA(2))
                nn =nn+1;
                Sys_tmp.A(n) =startpoints(nn+m);
            end
        elseif sA(1) == 1 &&  FitOpt.sameHFcoupling == true
            for n=1:(sA(2))
                nn = nn+1;
                Sys_tmp.A(:,n) = startpoints(nn+m);
            end
        elseif  sA(1) > 1 && FitOpt.isequivHFcoupling == true
            C = unique(FitOpt.equivHFcoupling);
            C_len = length(C);
            ts = 1;
            for n=1:C_len
                s = sum(FitOpt.equivHFcoupling(:) == n);
                for t=1:(sA(2))
                    nn = nn+1;
                    Sys_tmp.A(ts:ts+s-1,t)= startpoints(nn+m);
                end
                ts = ts+s;
            end
        elseif sA(1) > 1 && FitOpt.isequivHFcoupling == false
            for n=1:(sA(1))
                for k=1:(sA(2))
                    nn = nn+1;
                    Sys_tmp.A(n,k) = startpoints(nn+m);
                end
            end
        end
    end
    
    % Fitting experimental parameters
    if spinSys==1
        parameters = {'Temperature', 'mwFreq', 'ModAmp', 'mwPhase'};
        for k = 1:length(Exp)
            Exp_tmp = Exp{k};
            for p = 1:length(parameters)
                if isfield(Vary_tmp, parameters{p})
                    sA = size(Vary_tmp.(parameters{p}));
                    if sA(1) == 1
                        if  k > 1
                            nn = nn-(sA(1)*sA(2))-1;
                        end
                        for n=1:(sA(1)*sA(2))
                            nn = nn+1;
                            Exp_tmp.(parameters{p})(n) = startpoints(nn+m);
                        end
                    elseif sA(1) >1
                        for t=1:(sA(2))
                            nn = nn+1;
                            Exp_tmp.(parameters{p})(1,t) = startpoints(nn+m);
                        end
                    end
                end
            end
            Exp{k} = Exp_tmp;
        end
    end
    
    %Isotropic restriction in the fit
    Sys_tmp =  restrictiso(Sys_tmp, FitOpt);
    m = nn+m;
    Sys{spinSys} = Sys_tmp;
    Vary{spinSys} = Vary_tmp;
end

% Simulation kernel
[simfrequency_tmp, fitFunction_tmp] = simulation_kernel(easyspinfun,Sys, ...
                                                        Exp, FitOpt, ...
                                                        Opt,FitOpt.weighting,...
                                                        number_of_spc);
%% PLOT THE RESULTS
for y= 1:number_of_spc    
    % For Parallized gradient calculation catch plotting error
    if FitOpt.plotoff == false
        try
            refreshPlot(h{y},fitFunction_tmp{y},simfrequency_tmp{y});
        catch ME
        end
    end
end

%Concatenate the fit functions of a global fit
fitFunction = [fitFunction_tmp{1:length(Exp)}];
%simfrequency = [simfrequency_tmp{1:length(Exp)}];
end