function Signal = idiscrete_wt(trend, detail, varargin)

% Inverse Discrete wavelet decompostion on a dyadic scale
%
%   INPUT:
%   Signal                         - Vector
%                                    trend part of the DWT 
%
%   detail                         - Integer
%                                    detail part of the DWT 
%
%   OPTIONAL:
%   family                         - Integer
%                                    defines the wavelet family.
%
%   OUTPUT:
%   Signal                         - Vector 
%                                    Signal after inverse DWT 
%
%
% Inverse Discrete wavelet transforms are carried out using upsampling with
% subsequent convolutionwith the inverse (they are orthogonal) wavelet and
% (defined by family). The following wavelet families are available:
%
%   method               family
%     1              Daubechies D6-Wavelet 
%     2              Daubechies D10-Wavelet 
%     3              Daubechies D20-Wavelet       
%     4              Symlets 10-Wavelet
%     5              Coiflets 4-Wavelet
%     6              Haar-Wavelet
%
% (c) 2017, Stephan Rein, University of Freiburg

%Define wavelet family
if nargin > 1
    family = varargin{1};
else
    family = 1;
end

%Load the wavelet parameters
wavelet_parameters;

if family == 1
    h = db6f;
    g = db6m;
elseif family == 2
    h = db10f;
    g = db10m;
elseif family == 3
    h = db20f;
    g = db20m;
elseif family == 4
    h = sym10f;
    g = sym10m;
elseif family == 5
    h = coif4f;
    g = coif4m;
elseif family == 6
    h = haarf;
    g = haarm;
else
    h = db6f;
    g = db6m;
end


%Upsampling
S1 = upsamp(trend,2);
S2 = upsamp(detail,2);


%Convolution without zero padding
Conv1 = conv(S1,h,'same');
Conv2 = conv(S2,g,'same');

Signal = Conv2 + Conv1;
return
