function make_final_plot(FitOpt, Exp, Opt, easyspinfun, bestparameter, ...
    number_of_spc, lsqvalues, outputstr)

%% Make final plot if wished
if FitOpt.final_plot == true
    Sys = bestparameter;
    [FitOpt.Frequency_exp_tmp, FitOpt.Int_tmp] = simulation_kernel(easyspinfun,Sys, Exp, FitOpt, ...
        Opt,...
        FitOpt.weighting,number_of_spc);
    %make plot for the output
    for b =1:number_of_spc
        try
            figure(b+100)
            hold on
            FitOpt.Intexp_static{b} = FitOpt.Intexp_static{b}.*FitOpt.weighting{b};
            plot(FitOpt.Frequency_exp_tmp{b},FitOpt.Intexp_static{b})
            plot(FitOpt.Frequency_exp_tmp{b},FitOpt.Int_tmp{b},'r')
            defmax = max([max(FitOpt.Int_tmp{b}), max(FitOpt.Intexp_static{b})]);
            defmin = min([min(FitOpt.Int_tmp{b}), min(FitOpt.Intexp_static{b})]);
            axis([min(FitOpt.Frequency_exp_tmp{b}) max(FitOpt.Frequency_exp_tmp{b}) defmin-abs(defmin/8) defmax+abs(defmax/8)]);
            set(gca,'YTick',[])
            box on
            xlabel(FitOpt.xlab{b});
            ylabel('{\it Normalized Intensity}');
            set(b+100,'Units','centimeters','PaperSize',[7.05 7.05/sqrt(2)])
            set(b+100,'PaperOrientation','portrait')
            set(b+100,'PaperPosition',[0 0 get(b+100,'PaperSize')])
            
            % Improve font sizes
            textfontsize = 9;
            small = [gca ];%#ok<NBRAK> %hLeg];%
            normal = [get(gca,'XLabel') get(gca,'YLabel')];%
            set(small,  'FontSize',round(.8*textfontsize), 'FontName','Arial')
            set(normal, 'FontSize',round(.8*textfontsize), 'FontName','Arial')
            
            figure(b+200)
            hold on
            x_tmp = 1:FitOpt.maxn_iter;
            lsqvalues_tmp = sort(lsqvalues);
            plot(1:FitOpt.maxn_iter,lsqvalues_tmp,'o','MarkerSize',1.2)
            defmax = max(lsqvalues);
            defmin = min(lsqvalues);
            axis([0 max(x_tmp)+1 defmin-abs(defmin/8) defmax+abs(defmin/8)]);
            box on
            xlabel('{\it Optimisation}');
            ylabel('{\it \chi^2}');
            set(b+200,'Units','centimeters','PaperSize',[7.05 7.05/sqrt(2)])
            set(b+200,'PaperOrientation','portrait')
            set(b+200,'PaperPosition',[0 0 get(b+200,'PaperSize')])
            
            % Improve font sizes
            textfontsize = 9;
            small = gca;
            normal = [get(gca,'XLabel') get(gca,'YLabel')];%
            set(small,  'FontSize',round(.8*textfontsize), 'FontName','Arial')
            set(normal, 'FontSize',round(.8*textfontsize), 'FontName','Arial')
            
            % Improve font sizes
            textfontsize = 9;
            small = gca;
            normal = [get(gca,'XLabel') get(gca,'YLabel')];%
            set(small,  'FontSize',round(.8*textfontsize), 'FontName','Arial')
            set(normal, 'FontSize',round(.8*textfontsize), 'FontName','Arial')
            
            %% save figure as pdf
            str_tmp = strcat('Fit',num2str(b),'_',outputstr);
            print (b+100,'-dpdf', str_tmp);
            
            
            %% save figure as pdf and png
            str_tmp = strcat('Least_squares_',outputstr);
            print (b+200,'-dpdf', str_tmp);
            
            
            %% Save the data vector
            A = [FitOpt.Frequency_exp_tmp{b};FitOpt.Intexp_static{b};FitOpt.Int_tmp{b}];
            fileID = fopen(strcat(str_tmp,'.txt'),'w');
            fprintf(fileID,'%6.6f %12.8f %12.8f\n',A);
            fclose(fileID);
            close(figure(b+100))
            close(figure(b+200))
        catch ME
            warning('Execption raised. No final plots were created\n')
            fprintf(ME.identifier)
        end
    end
end
end